@@ -1,5 +1,4 @@
-ELECTRUM_VERSION = '4.5.0'       # version of the client package
-APK_VERSION = '4.5.0.1'          # read by buildozer.spec
+ELECTRUM_VERSION = '4.5.6'       # version of the client package
 
 PROTOCOL_VERSION = '1.4'     # protocol version requested
 
@@ -19,4 +18,4 @@ def seed_prefix(seed_type):
         return SEED_PREFIX_2FA
     elif seed_type == '2fa_segwit':
         return SEED_PREFIX_2FA_SW
-    raise Exception(f"unknown seed_type: {seed_type}")
+    raise Exception(f"unknown seed_type: {seed_type!r}")
